﻿# 2_update_users.ps1 - Updates user names, archive names and priviliges on archives
#                      according to the file created by the 1_prepare_users.ps1 script.
#
# Version 1.1: check for upper case letters in target user name added
#              check for write-protected archive stores added
# Version 1.0: initial version
#
# Copyright (c) 2017 - 2022 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore administrator, default is "admin"
$username = "admin"

# If password is empty, an interactive prompt will ask for it
$password = ''

# The hostname of the MailStore Server, default is "localhost"
$hostname = "localhost"

# TCP Port of the Management API, API must be enabled in the MailStore Server Service Configuration, default is 8463
$port = 8463

# Set this to $true when untrusted certificates are used, default is $false
$acceptAllCertificates = $false

# File where the user list is stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH "mailstore-users.txt")

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

if ($password -ceq ''){
    $password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Password for $username")))
}

function Has-Succeeded($result) {
    if ($result.statusCode -ne "succeeded") {
        Write-Host ("Status  " + $result.statusCode)
        Write-Host ("Message " + $result.error.message)
        Write-Host ("Details " + $result.error.details)
        return $false
    }
    return $true
}

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

if (-not (Test-Path $filename -PathType Leaf)){
    Write-Host ($filename + " does not exist.")
    My-Exit
}

Write-Host "Creating API client"
$msapiclient = New-MSApiClient -Username $username -Password $password -MailStoreServer $hostname -Port $port -IgnoreInvalidSSLCerts:$acceptAllCertificates

Write-Host "Checking for write-protected archive stores"
$archive_stores_result = Invoke-MSApiCall $msapiclient "GetStores"

if (-not (Has-Succeeded($archive_stores_result))) {
    My-Exit
}

if ($archive_stores_result.result.requestedState -match "writeProtected") {
    foreach ($archive_store in $archive_stores_result.result){
        if ($archive_store.requestedState -ceq "writeProtected") {
            Write-Host ("Archive store with ID: " + $archive_store.id + " Name: '" + $archive_store.name + "' is write protected")
        }
    }
    Write-Host ("At least one write-protected archive store has been found. Script aborting.")
    My-Exit
}

Write-Host ("Creating mapping from users file " + $filename)

$user_mapping = @{}

foreach ($line in (Get-Content $filename))
{
    # skip empty lines and lines starting with a #
    if ($line.StartsWith("#") -or -not $line) {
        Write-Host ("Skipping line '" + $line + "'")
        continue
    }

    $username_parts = $line.Split("=")
	
    # skip users which contain at least one equals character
	if ($username_parts.count -gt 2) {
		Write-Host ("Source user name or target user name contains at least one equals character (=). Line: {0}. Skipping" -f $line)
		continue
	}
	
    $source_user_name = $username_parts[0]
    $target_user_name = $username_parts[1]

    # skip users without source
    if (-not $source_user_name){
        Write-Host ("Source user name of '" + $target_user_name + "' is empty. Skipping")
        continue
    }

    # skip users without target
    if (-not $target_user_name){
        Write-Host ("Target user name of '" + $source_user_name + "' is empty. Skipping")
        continue
    }

    # skip users that do not change
    if ($source_user_name -eq $target_user_name){
        Write-Host ("Source user name and target user name are identical '" + $source_user_name + "'. Skipping")
        continue
    }

    # convert users where the target name contains upper case letters to lower case
    if ($target_user_name -cne $target_user_name.toLower()){
        Write-Host ("Target user name must be all lower case but is'" + $target_user_name + "'. Auto converted to lower case.")
        $target_user_name =  $target_user_name.toLower()
    }

    Write-Host ("User name '" + $source_user_name + "' will be renamed to '" + $target_user_name + "'")
    $user_mapping.Add($source_user_name, $target_user_name)
}

# check for duplicate target user names
$duplicate_target_user_names = ($user_mapping.values | Group-Object | Where-Object Count -gt 1).Values
if ($duplicate_target_user_names) {
    Write-Host ("Duplicate target user names found in users file: {0}" -f ($duplicate_target_user_names -join ", "))
    Write-Host ("Aborting.")
    My-Exit
}

Write-Host ("Fetching MailStore users from " + $username + "@" + $hostname + ":" + $port)

$mailstore_users_result = Invoke-MSApiCall $msapiclient "GetUsers"

if (Has-Succeeded($mailstore_users_result)){
    $mailstore_users = $mailstore_users_result.result.userName
}
else {
    My-Exit
}

Write-Host "Applying changes on existing users"
foreach ($user in $mailstore_users) {
    # skipping all operations for already existing target user names
    if ($mailstore_users -contains $user_mapping.$user) {
        Write-Host ("New user name of '"+ $user +"' already exists on server: " + $user_mapping.$user)
        continue
    }

    # Write-Host ("Reading folder permissions of " + $user_mapping.$user)

    $user_info_result = Invoke-MSApiCall $msapiclient "GetUserInfo" @{"userName" = $user}

    if (-not (Has-Succeeded($user_info_result))) {
        Write-Host ("Error while executing GetUserInfo of user " + $user + ". Skipping all other operations")
        continue
    }
    $user_info = $user_info_result.result

    # adjust folder privileges for regular users
    if (-not ($user_info.privileges -contains "admin")) {
        foreach ($privilege in $user_info.privilegesOnFolders) {
            $folder = $privilege.folder
            $priv = $privilege.privileges
            
            if ($user_mapping.Keys -contains $folder) {
                Write-Host ("Adding   permission. User: " + $user + " Archive: " + $user_mapping.$folder + " Privileges: " + ($priv -join ","))
                $privilege_change_result = Invoke-MSApiCall $msapiclient "SetUserPrivilegesOnFolder" @{"userName" = $user; "folder" = $user_mapping.$folder; "privileges" = ($priv -join ",")}
                
                if (-not (Has-Succeeded($privilege_change_result))) {
                    continue
                }
                
                # Write-Host $privilege_change_result.statusCode
                
                Write-Host ("Removing permission. User: " + $user + " Archive: " +  $folder)
                $privilege_delete_result = Invoke-MSApiCall $msapiclient "SetUserPrivilegesOnFolder" @{"userName" = $user; "folder" = $folder; "privileges" = "none"}
                
                if (-not (Has-Succeeded($privilege_delete_result))) {
                    continue
                }
                
                # Write-Host $privilge_delete_result.statusCode
            }
        }
    }

    # Rename users and archive
    # but skip renaming if user is not in mapping
    if ($user_mapping.Keys -contains $user) {
        # skip moving to the same name
        if ($user -ne $user_mapping.$user) {
            Write-Host ("Renaming user    " + $user + " to " + $user_mapping.$user)
            $rename_user_result = Invoke-MSApiCall $msapiclient "RenameUser" @{"oldUserName" = $user; "newUserName" = $user_mapping.$user}
    
            if (-not (Has-Succeeded($rename_user_result))) {
                continue
            }
            # Write-Host $rename_user_result.statusCode

            Write-Host ("Renaming archive " + $user + " to " + $user_mapping.$user)

            $rename_folder_result = Invoke-MSApiCall $msapiclient "MoveFolder" @{"fromFolder" = $user; "toFolder" = $user_mapping.$user}

            if (-not (Has-Succeeded($rename_folder_result))) {
                continue
            }
            # Write-Host $rename_folder_result.statusCode
        }
    }
} 

Write-Host ("Fetching updated MailStore archives from " + $username + "@" + $hostname + ":" + $port)
$mailstore_archives_result = Invoke-MSApiCall $msapiclient "GetChildFolders" @{"maxLevels" = 1}

if (Has-Succeeded($mailstore_archives_result)){
    $mailstore_archives = $mailstore_archives_result.result.childFolders.name
}
else {
    My-Exit
}

Write-Host "Applying changes on remaining archives"
foreach ($archive in $user_mapping.Keys) {
    # skipping all operations for already existing target archive names
    # could have been renamed in the previous step
    # avoid this to prevent unintended merges which cannot be rolled back easily
    if ($mailstore_archives -contains $user_mapping.$archive) {
        Write-Host ("New archive name of '"+ $archive +"' already exists on server: " + $user_mapping.$archive + " Skipping")
        continue
    }

    # Rename archive
    Write-Host ("Renaming archive " + $archive + " to " + $user_mapping.$archive)
    $rename_archive_result = Invoke-MSApiCall $msapiclient "MoveFolder" @{"fromFolder" = $archive; "toFolder" = $user_mapping.$archive}
    if (-not (Has-Succeeded($rename_archive_result))) {
        continue
    }
    # Write-Host $rename_archive_result.statusCode
} 

   
My-Exit
